"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// private properties
const componentsByTagName = new Map();
// public function
exports.register = (tagDef, mode = 'open') => __awaiter(void 0, void 0, void 0, function* () {
    if (!customElements) {
        throw new Error('Custom elements is not supported by your browser!');
    }
    if (tagDef.templateUrl) {
        const response = yield fetch(tagDef.templateUrl);
        const div = document.createElement('div');
        div.innerHTML = yield response.text();
        tagDef.template = div.firstChild;
        if (!tagDef.template.content) {
            throw (`Failed to create template from\n${div.innerHTML}`);
        }
    }
    else if (!tagDef.template) {
        throw ('templateUrl or template must defined. They can not both be blank.');
    }
    ;
    const wrapper = class Wrapper extends HTMLElement {
        constructor() {
            // Always call super first in constructor
            super();
            // Create a shadow root
            const shadow = this.attachShadow({ mode: mode });
            // create our component
            const component = tagDef.tagFunction();
            const clone = document.importNode(tagDef.template.content, true);
            component.element = clone.querySelector('[data-component-root="root"]');
            // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
            // @ts-ignore
            const styles = [...tagDef.template.content.childNodes].find((child) => {
                if (child.nodeType === Node.TEXT_NODE) {
                    return false;
                }
                return child.tagName.toLowerCase() === 'style';
            });
            if (!componentsByTagName.get(tagDef.tagName)) {
                componentsByTagName.set(tagDef.tagName, []);
            }
            // store the component instances by tag name so an observer can assign mediators
            // for a surrounding application
            componentsByTagName.get(tagDef.tagName).push(component);
            const renderedComponent = component.render();
            // TODO add lifecycle hooks
            // Attach the created elements to the shadow dom
            if (styles) {
                const style = document.createElement('style');
                style.textContent = styles.textContent;
                shadow.appendChild(style);
            }
            shadow.appendChild(renderedComponent);
            this.component = component;
        }
        connectedCallback() {
            console.log(`tag ${tagDef.tagName} inserted`);
            tagDef.inserted(this.component);
        }
        disconnectedCallback() {
            console.log(`tag ${tagDef.tagName} removed`);
            // remove the component instance
            const index = componentsByTagName.get(tagDef.tagName).findIndex((instance) => instance === this.component);
            componentsByTagName.get(tagDef.tagName).splice(index, 1);
            // destroy the component, this removes event listeners etc
            this.component.destroy(true);
            tagDef.removed(this.component);
            tagDef.template = null;
        }
    };
    customElements.define(tagDef.tagName, wrapper);
});
// this function is used by the surrounding application to mediate component instances
// generally a mediator map of some kind is created that will iterate over all registered
// tag name, retrieve the component instance for this function, and pass to the registered mediator constructor
// or function. A mediator map calls a function or constructor for every instance of a tag
// found in the DOM.. For example map('lotus-button', createButtonMediator). This map function
// would use getComponents to get all the component instances which it would then iterate over
// and call createButtonMediator(component). This function is also called by collection components
// such as image galleries or data grids. Passing the optional parent param allows collections
// to get all child component instances to assign event listeners etc.
exports.getComponents = (tagName, parent) => {
    return componentsByTagName.get(tagName).filter((component) => (parent) ? parent.contains(component.element) : true);
};
//# sourceMappingURL=ComponentRegistry.js.map