"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractComponent_1 = require("./AbstractComponent");
const __1 = require("../..");
// export public functions
exports.createItemView = (component) => {
    const clone = AbstractComponent_1.mixin(component, {
        model: null,
        resetState: (selected) => {
            if (selected) {
                clone.element.classList.add('selected');
            }
            else {
                clone.element.classList.remove('selected');
            }
        },
    });
    const destroy = clone.destroy;
    const render = clone.render;
    clone.destroy = () => {
        destroy();
    };
    clone.render = (list) => {
        render(list);
        return clone.element;
    };
    return clone;
};
exports.createComponent = (component) => {
    // TODO figure out which of the functions below (if any) should be private
    let clone = AbstractComponent_1.mixin(component, {
        addChildView: null,
        childViews: [],
        addViewEventListeners: null,
        addViewEventListener: null,
        removeViewEventListener: null,
        cloneItemTemplate: null,
        createChildView: null,
        onItemSelectedDeselect: null,
        removeViewEventListeners: null,
        selectedItem: null,
        removeAllChildViews: null,
        destroyChildViews: null,
        removeElement: null,
        removeChildView: null,
    });
    const render = clone.render;
    const views = [];
    let selectedItem;
    clone.cloneItemTemplate = () => {
        return clone.skinPartMap.get('itemTemplate').cloneNode(true);
    };
    clone.onItemSelectedDeselect = (event) => {
        if (selectedItem && selectedItem != event.payload['item']) {
            selectedItem.resetState(false);
        }
        selectedItem = event.payload['item'];
        selectedItem.resetState(event.type === AbstractComponent_1.Events.ITEM_SELECTED);
    };
    clone.addViewEventListeners = () => {
        __1.getComponents(clone.skinPartMap.get('itemTemplate').tagName.toLowerCase(), clone.skinPartMap.get('collectionContainer')).forEach((view) => {
            // components removed from the DOM are automatically removed from the results of getComponents
            // So we store our own ref in views so we can remove listener functions later
            views.push(view);
            clone.addViewEventListener(view);
        });
    };
    clone.addViewEventListener = (view) => {
        view.addEventListener(AbstractComponent_1.Events.ITEM_SELECTED, clone, 'onItemSelectedDeselect');
        view.addEventListener(AbstractComponent_1.Events.ITEM_DESELECTED, clone, 'onItemSelectedDeselect');
        view.addEventListener(AbstractComponent_1.Events.REMOVE_ITEM, clone, 'onItemRemove');
    };
    clone.removeViewEventListener = (view) => {
        view.removeEventListener(AbstractComponent_1.Events.ITEM_SELECTED, clone, 'onItemSelectedDeselect');
        view.removeEventListener(AbstractComponent_1.Events.ITEM_DESELECTED, clone, 'onItemSelectedDeselect');
        view.removeEventListener(AbstractComponent_1.Events.REMOVE_ITEM, clone, 'onItemRemove');
    };
    clone.removeViewEventListeners = () => {
        views.forEach((view) => {
            clone.removeViewEventListener(view);
        });
    };
    clone.addChildView = (model) => {
        // TODO: replace clone.createChildView with a ComponentRegistry method
        const element = clone.cloneItemTemplate();
        clone.skinPartMap.get('collectionContainer').appendChild(element);
        // IMPORTANT: once the component is added to the DOM the ComponentRegistry
        // assigns the component attribute
        // render the component again passing the model
        element.replaceWith(element['component'].render([model]));
    };
    clone.removeAllChildViews = () => {
        const node = clone.skinPartMap.get('collectionContainer');
        while (node.hasChildNodes()) {
            node.removeChild(node.lastChild);
        }
    };
    clone.destroy = (removed = false) => {
        if (!clone.ready) {
            return;
        }
        clone.removeViewEventListeners();
        clone.removeAllChildViews();
        views.splice(0, views.length);
        clone.removeEventListeners();
        const itemTemplate = clone.skinPartMap.get('itemTemplate');
        clone.attributeMap.clear();
        clone.skinPartMap.clear();
        // the item template is removed from the DOM on load so we need to make sure we preserve a reference
        clone.skinPartMap.set('itemTemplate', itemTemplate);
        // only make clone eligible for GC if removed from DOM
        if (removed) {
            clone = null;
        }
    };
    clone.render = (list) => {
        // call super, triggers destroy
        render(list);
        // render can be called by the ComponentRegistry as part of lifecycle
        if (!list) {
            return clone.element;
        }
        // SSR example, navigator.webdriver denote puppeteer agent
        // when rendering client side we detect child view using the itemTemplate
        if (navigator.webdriver ||
            __1.getComponents(clone.skinPartMap.get('itemTemplate').tagName.toLowerCase(), clone.skinPartMap.get('collectionContainer')).length === 0) {
            list.forEach((model) => {
                clone.addChildView(model);
            });
        }
        clone.addViewEventListeners();
        return clone.element;
    };
    clone.onSkinPartAdded = (part) => {
        switch (part) {
            // required, defines the layout for child views
            case 'itemTemplate':
                clone.skinPartMap.get(part).remove(); // remove from the view
                break;
        }
    };
    AbstractComponent_1.addProperty(clone, 'selectedItem', function () {
        return selectedItem;
    });
    return clone;
};
//# sourceMappingURL=AbstractCollectionComponent.js.map