export interface ComponentEvent {
    type: string;
    payload: object;
    clone(event: ComponentEvent): ComponentEvent;
}
export interface EventDispatcher {
    handlersByEventName: Map<string, Array<Listener>>;
    addEventListener(event: string, instance: object, handler: string): void;
    canListen(eventType: string, instance: object, handler: string): boolean;
    removeEventListener(event: string, instance: object, handler: string): void;
    removeAllEventListeners(instance: object): void;
    dispatch(event: ComponentEvent): void;
}
export interface Component extends EventDispatcher {
    ready: boolean;
    id: string;
    element: HTMLElement;
    destroy(removed?: boolean): void;
    init(): void;
    inserted(): void;
    removed(): void;
    attributeChanged(): void;
    removeEventListeners(): void;
    onSkinPartAdded(part: string): void;
    attributeMap: Map<string, unknown>;
    skinPartMap: Map<string, HTMLElement>;
    addAttributes(): void;
    addSkinParts(): void;
    render<T>(list?: Array<T>): HTMLElement;
}
export interface Listener {
    readonly handler: string;
    readonly instance: object;
}
export declare const random: () => string;
export declare enum Events {
    CLICK = "lotusComponentClick",
    ITEM_SELECTED = "lotusItemViewItemSelected",
    ITEM_DESELECTED = "lotusItemViewItemDeselected",
    REMOVE_ITEM = "lotusItemViewRemoveItem"
}
export declare const addProperty: <T>(instance: T, label: string, getter?: () => unknown, setter?: (v: unknown) => void, enumerable?: boolean) => T;
export declare const mixin: <T>(target: any, sub: any) => T;
export declare const getTemplate: <T extends Component>() => T;
export declare const getComponentEvent: (type: string, payload: object) => ComponentEvent;
export declare const createComponent: () => Component;
