"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractComponent_1 = require("./AbstractComponent");
const AbstractCollectionComponent_1 = require("./AbstractCollectionComponent");
const compose_1 = require("ramda/es/compose");
exports.getScaleToFit = (objSize, sizeToFit) => {
    return Math.min(sizeToFit.width / objSize.width, sizeToFit.height / objSize.height);
};
// export public functions
exports.createImageView = (component) => {
    // TODO figure out which of the functions below (if any) should be private
    const clone = AbstractComponent_1.mixin(component, {
        width: NaN,
        height: NaN,
        allowDrag: true,
        onThumbClick: false,
        onDragStart: false,
        onImageLoad: false,
        setThumbnailSrc: false,
    });
    const render = clone.render;
    const destroy = clone.destroy;
    const onSkinPartAdded = clone.onSkinPartAdded;
    const resetState = clone.resetState;
    const sizeImage = () => {
        if (!clone.skinPartMap.get('thumbnail')) {
            return;
        }
        const imageSize = {
            width: parseInt(clone.skinPartMap.get('thumbnail').getAttribute('width')),
            height: parseInt(clone.skinPartMap.get('thumbnail').getAttribute('height')),
        };
        const containerSize = {
            width: parseInt(window.getComputedStyle(clone.element).width),
            height: parseInt(window.getComputedStyle(clone.element).height),
        };
        const scale = exports.getScaleToFit(imageSize, containerSize);
        const width = imageSize.width * scale;
        const height = imageSize.height * scale;
        // console.log("width/height "+width+"/"+height)
        clone.skinPartMap.get('thumbnail').setAttribute('width', `${width}px`);
        clone.skinPartMap.get('thumbnail').setAttribute('height', `${height}px`);
        clone.skinPartMap.get('thumbnail').style.maxWidth = `${containerSize.width}px`;
        clone.skinPartMap.get('thumbnail').style.maxHeight = `${containerSize.height}px`;
    };
    clone.onThumbClick = () => {
        clone.resetState(clone.element.classList.contains('selected'));
    };
    clone.onDragStart = () => {
        // TODO: attach data attribute
    };
    clone.resetState = (state) => {
        resetState(state);
        console.log(`you clicked: ${clone.skinPartMap.get('caption').innerHTML}`);
        // TODO: handle overrides
    };
    clone.onImageLoad = () => {
        clone.skinPartMap.get('thumbnail').onload = null;
        clone.skinPartMap.get('thumbnail').style.visibility = 'visible';
        sizeImage();
    };
    clone.setThumbnailSrc = (src) => {
        clone.skinPartMap.get('thumbnail').onload = clone.onImageLoad;
        clone.skinPartMap.get('thumbnail').style.visibility = 'hidden';
        clone.skinPartMap.get('thumbnail')['src'] = src;
    };
    clone.removeEventListeners = () => {
        clone.skinPartMap.get('thumbnail').removeEventListener('click', clone.onThumbClick);
        clone.skinPartMap.get('thumbnail').removeEventListener('dragstart', clone.onDragStart);
    };
    clone.onSkinPartAdded = (part) => {
        onSkinPartAdded(part);
        switch (part) {
            // required, defines the layout for child views
            case 'thumbnail':
                clone.skinPartMap.get('thumbnail').setAttribute('draggable', (clone.allowDrag) ? 'true' : 'false');
                clone.skinPartMap.get('thumbnail').addEventListener('click', clone.onThumbClick);
                clone.skinPartMap.get('thumbnail').addEventListener('dragstart', clone.onDragStart);
                break;
        }
    };
    clone.destroy = () => {
        if (!clone.ready) {
            return;
        }
        // IMPORTANT child components need to clean up before calling super.destroy!!!
        clone.removeEventListeners();
        destroy();
    };
    clone.render = (list) => {
        render(list);
        if (list && list.length === 1) {
            clone.setThumbnailSrc(list[0].src);
            if (clone.skinPartMap.get('caption')) {
                clone.skinPartMap.get('caption').innerHTML = list[0].caption;
            }
        }
        return clone.element;
    };
    return clone;
};
exports.createComponent = (component) => {
    return AbstractComponent_1.mixin(component, {
        title: ''
    });
};
// create hook using compose
// hooks provide prebuilt functions that are useful
exports.useImageView = compose_1.default(exports.createImageView, AbstractCollectionComponent_1.createItemView, AbstractComponent_1.createComponent);
exports.useImageGallery = compose_1.default(exports.createComponent, AbstractCollectionComponent_1.createComponent, AbstractComponent_1.createComponent);
//# sourceMappingURL=ImageGallery.js.map